@@ -1,13 +1,13 @@
 /*
  * rb-audioscrobbler-plugin.c
- * 
+ *
  * Copyright (C) 2006 James Livingston <jrl@ids.org.au>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -30,6 +30,7 @@
 #include <glib-object.h>
 
 #include "rb-audioscrobbler.h"
+#include "rb-lastfm-source.h"
 #include "rb-plugin.h"
 #include "rb-debug.h"
 #include "rb-shell.h"
@@ -48,6 +49,9 @@ typedef struct
 	RBPlugin parent;
 	RBAudioscrobbler *audioscrobbler;
 	GtkWidget *preferences;
+	guint ui_merge_id;
+
+	RBSource *lastfm_source;
 } RBAudioscrobblerPlugin;
 
 typedef struct
@@ -111,15 +115,23 @@ impl_activate (RBPlugin *bplugin,
 {
 	RBAudioscrobblerPlugin *plugin = RB_AUDIOSCROBBLER_PLUGIN (bplugin);
 	RBProxyConfig *proxy_config;
+	GtkUIManager *uimanager = NULL;
 	gboolean no_registration;
+	char *file;
+
+    /* Source icon data */
+    gchar *icon_filename;
+    gint icon_size;
+    GdkPixbuf *icon_pixbuf;
 
 	g_assert (plugin->audioscrobbler == NULL);
-	g_object_get (G_OBJECT (shell), 
-		      "proxy-config", &proxy_config, 
+	g_object_get (G_OBJECT (shell),
+		      "proxy-config", &proxy_config,
 		      "no-registration", &no_registration,
+		      "ui-manager", &uimanager,
 		      NULL);
 
-	/* 
+	/*
 	 * Don't use audioscrobbler when the no-registration flag is set.
 	 * This flag is only used to run multiple instances at the same time, and
 	 * last.fm only allows one active client per user.
@@ -129,6 +141,27 @@ impl_activate (RBPlugin *bplugin,
 								proxy_config);
 	}
 	g_object_unref (G_OBJECT (proxy_config));
+
+	file = rb_plugin_find_file (bplugin, "audioscrobbler-ui.xml");
+	plugin->ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager,
+							       file,
+							       NULL);
+	g_free (file);
+
+	plugin->lastfm_source = rb_lastfm_source_new (shell);
+    
+    icon_filename = rb_plugin_find_file (bplugin, "as-icon.png");
+    gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &icon_size, NULL);
+    icon_pixbuf = gdk_pixbuf_new_from_file_at_size (icon_filename,
+                                                    icon_size, icon_size,
+                                                    NULL);
+    g_free (icon_filename);
+    rb_source_set_pixbuf (plugin->lastfm_source, icon_pixbuf);
+    g_object_unref (icon_pixbuf);
+    
+	rb_shell_append_source (shell, plugin->lastfm_source, NULL);
+
+	g_object_unref (G_OBJECT (uimanager));
 }
 
 static void
@@ -136,6 +169,17 @@ impl_deactivate	(RBPlugin *bplugin,
 		 RBShell *shell)
 {
 	RBAudioscrobblerPlugin *plugin = RB_AUDIOSCROBBLER_PLUGIN (bplugin);
+	GtkUIManager *uimanager = NULL;
+
+	g_object_get (G_OBJECT (shell),
+		      "ui-manager", &uimanager,
+		      NULL);
+
+	rb_source_delete_thyself (plugin->lastfm_source);
+	plugin->lastfm_source = NULL;
+
+	gtk_ui_manager_remove_ui (uimanager, plugin->ui_merge_id);
+	g_object_unref (G_OBJECT (uimanager));
 
 	if (plugin->audioscrobbler) {
 		g_object_unref (plugin->audioscrobbler);
@@ -159,26 +203,22 @@ impl_create_configure_dialog (RBPlugin *bplugin)
 	if (plugin->preferences == NULL) {
 		GtkWidget *widget;
 
-		widget =  rb_audioscrobbler_get_config_widget (plugin->audioscrobbler);
+		widget =  rb_audioscrobbler_get_config_widget (plugin->audioscrobbler, bplugin);
 
-		plugin->preferences = gtk_dialog_new_with_buttons (_("Last.fm Profile Preferences"),
+		plugin->preferences = gtk_dialog_new_with_buttons (_("Audioscrobbler preferences"),
 								   NULL,
 								   GTK_DIALOG_DESTROY_WITH_PARENT,
 								   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 								   NULL);
-		gtk_container_set_border_width (GTK_CONTAINER (plugin->preferences), 5);
-		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (plugin->preferences)->vbox), 2);
-		gtk_dialog_set_has_separator (GTK_DIALOG (plugin->preferences), FALSE);
-		
 		g_signal_connect (G_OBJECT (plugin->preferences),
 				  "response",
 				  G_CALLBACK (preferences_response_cb),
 				  plugin);
 		gtk_widget_hide_on_delete (plugin->preferences);
-		
+
 		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (plugin->preferences)->vbox), widget);
 	}
-	
+
 	gtk_widget_show_all (plugin->preferences);
 	return plugin->preferences;
 }
