@@ -1,7 +1,7 @@
 /*
  *  arch-tag: Implementation of ipod source object
  *
- *  Copyright (C) 2004 Christophe Fergeau  <teuf@gnome.org>
+ *  Copyright (C) 2004, 2007 Christophe Fergeau  <teuf@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,9 +21,11 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #ifdef HAVE_HAL
 #include <libhal.h>
@@ -36,6 +38,7 @@
 
 #include "eel-gconf-extensions.h"
 #include "rb-ipod-source.h"
+#include "rb-ipod-db.h"
 #include "rb-debug.h"
 #include "rb-file-helpers.h"
 #include "rb-plugin.h"
@@ -45,11 +48,6 @@
 #include "rhythmdb.h"
 #include "rb-cut-and-paste-code.h"
 
-#ifdef IPOD_SUPPORT
-#define PHONE_VENDOR_ID 0x22b8
-#define PHONE_PRODUCT_ID 0x4810
-#endif
-
 static GObject *rb_ipod_source_constructor (GType type,
 					    guint n_construct_properties,
 					    GObjectConstructParam *construct_properties);
@@ -62,7 +60,6 @@ static void rb_ipod_source_dispose (GObject *object);
 static gboolean impl_show_popup (RBSource *source);
 static void impl_move_to_trash (RBSource *asource);
 static void rb_ipod_load_songs (RBiPodSource *source);
-static gchar *rb_ipod_get_mount_path (GnomeVFSVolume *volume);
 static void impl_delete_thyself (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
 #ifdef HAVE_HAL
@@ -86,18 +83,29 @@ static gchar* ipod_get_filename_for_uri (const gchar *mount_point,
 static gchar* ipod_path_from_unix_path (const gchar *mount_point,
 					const gchar *unix_path);
 #endif
-static void itdb_schedule_save (RBiPodSource *source);
+static RhythmDB *get_db_for_source (RBiPodSource *source);
+
+struct _PlayedEntry {
+	RhythmDBEntry *entry;
+	guint play_count;
+};
+
+typedef struct _PlayedEntry PlayedEntry;
 
 typedef struct
 {
-	Itdb_iTunesDB *ipod_db;
-	gchar *ipod_mount_path;
+	RbIpodDb *ipod_db;
 	GHashTable *entry_map;
 
-	GList *playlists;
 	gboolean needs_shuffle_db;
+	RBStaticPlaylistSource *podcast_pl;
 
 	guint load_idle_id;
+
+	GHashTable *artwork_request_map;
+	guint artwork_notify_id;
+
+	GQueue *offline_plays;
 } RBiPodSourcePrivate;
 
 RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
@@ -118,8 +126,8 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_delete_thyself = impl_delete_thyself;
-   	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
-   	source_class->impl_move_to_trash = impl_move_to_trash;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_move_to_trash = impl_move_to_trash;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
 
@@ -136,27 +144,13 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	g_type_class_add_private (klass, sizeof (RBiPodSourcePrivate));
 }
 
+
 static void
 rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
 {
-	Itdb_Playlist *mpl;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
-	mpl = itdb_playlist_mpl (priv->ipod_db);
-	if (mpl != NULL) {
-		if (mpl->name != NULL) {
-			rb_debug ("Renaming iPod from %s to %s", mpl->name, name);
-			if (strcmp (mpl->name, name) == 0) {
-				rb_debug ("iPod is already named %s", name);
-				return;
-			}
-		}
-		g_free (mpl->name);
-		mpl->name = g_strdup (name);
-		itdb_schedule_save (source);
-	} else {
-		g_warning ("iPod's master playlist is missing");
-	}
+	rb_ipod_db_set_ipod_name (priv->ipod_db, name);
 }
 
 static void
@@ -172,7 +166,7 @@ rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 
 static void
 rb_ipod_source_init (RBiPodSource *source)
-{
+{	
 	g_signal_connect (G_OBJECT (source), "notify::name",
 			  (GCallback)rb_ipod_source_name_changed_cb, NULL);
 }
@@ -183,25 +177,13 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 {
 	RBiPodSource *source;
 	RBEntryView *songs;
-	RBiPodSourcePrivate *priv;
-	const Itdb_IpodInfo *info;
 
 	source = RB_IPOD_SOURCE (G_OBJECT_CLASS (rb_ipod_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
-	priv = IPOD_SOURCE_GET_PRIVATE (source);
-
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
 
-	info = itdb_device_get_ipod_info(priv->ipod_db->device);
-	if (info->ipod_generation == ITDB_IPOD_GENERATION_UNKNOWN ||
-	    info->ipod_model == ITDB_IPOD_MODEL_SHUFFLE) {
-		priv->needs_shuffle_db = TRUE;
-	} else {
-		priv->needs_shuffle_db = FALSE;
-	}
-
 	rb_ipod_load_songs (source);
 
 	return G_OBJECT (source);
@@ -212,14 +194,9 @@ rb_ipod_source_dispose (GObject *object)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (object);
 
- 	if (priv->ipod_db != NULL) {
- 		itdb_free (priv->ipod_db);
- 		priv->ipod_db = NULL;
-  	}
-
-	if (priv->ipod_mount_path) {
-		g_free (priv->ipod_mount_path);
-		priv->ipod_mount_path = NULL;
+	if (priv->ipod_db) {
+		g_object_unref (G_OBJECT (priv->ipod_db));
+		priv->ipod_db = NULL;
 	}
 
 	if (priv->entry_map) {
@@ -232,6 +209,25 @@ rb_ipod_source_dispose (GObject *object)
 		priv->load_idle_id = 0;
 	}
 
+	if (priv->artwork_request_map) {
+		g_hash_table_destroy (priv->artwork_request_map);
+		priv->artwork_request_map = NULL;
+ 	}
+
+	if (priv->artwork_notify_id) {
+		RhythmDB *db = get_db_for_source (RB_IPOD_SOURCE (object));
+		g_signal_handler_disconnect (db, priv->artwork_notify_id);
+		priv->artwork_notify_id = 0;
+		g_object_unref (db);
+	}
+
+	if (priv->offline_plays) {
+		g_queue_foreach (priv->offline_plays,
+				 (GFunc)g_free, NULL);
+		g_queue_free (priv->offline_plays);
+		priv->offline_plays = NULL;
+	}
+
 	G_OBJECT_CLASS (rb_ipod_source_parent_class)->dispose (object);
 }
 
@@ -258,11 +254,11 @@ rb_ipod_source_new (RBShell *shell,
 	g_free (path);
 
 	source = RB_IPOD_SOURCE (g_object_new (RB_TYPE_IPOD_SOURCE,
-					  "entry-type", entry_type,
-					  "volume", volume,
-					  "shell", shell,
-					  "sourcelist-group", RB_SOURCELIST_GROUP_REMOVABLE,
-					  NULL));
+					       "entry-type", entry_type,
+					       "volume", volume,
+					       "shell", shell,
+					       "source-group", RB_SOURCE_GROUP_DEVICES,
+					       NULL));
 
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
 
@@ -300,6 +296,46 @@ ipod_path_to_uri (const char *mount_point, const char *ipod_path)
  	return uri;
 }
 
+static void
+playlist_track_removed (RhythmDBQueryModel *m,
+			RhythmDBEntry *entry,
+			gpointer data)
+{
+	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
+	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+	Itdb_Track *track;
+
+	g_return_if_fail (ipod != NULL);
+	g_return_if_fail (ipod_pl != NULL);
+
+        track = g_hash_table_lookup (priv->entry_map, entry);
+	g_return_if_fail (track != NULL);
+	rb_ipod_db_remove_from_playlist (priv->ipod_db, ipod_pl, track);
+}
+
+static void
+playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
+		      GtkTreeIter *iter, gpointer data)
+{
+	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
+	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+	Itdb_Track *track;
+	RhythmDBEntry *entry;
+
+	g_return_if_fail (ipod != NULL);
+	g_return_if_fail (ipod_pl != NULL);
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+        track = g_hash_table_lookup (priv->entry_map, entry);
+	g_return_if_fail (track != NULL);
+
+	rb_ipod_db_add_to_playlist (priv->ipod_db, ipod_pl, track);
+}
+
 static void
 add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 {
@@ -308,11 +344,12 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
+	RhythmDBQueryModel *model;
 
-  	g_object_get (source,
-		      "shell", &shell,
-		      "entry-type", &entry_type,
-		      NULL);
+	g_object_get (source,
+			  "shell", &shell,
+			  "entry-type", &entry_type,
+			  NULL);
 
 	playlist_source = rb_static_playlist_source_new (shell,
 							 playlist->name,
@@ -323,17 +360,35 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	for (it = playlist->members; it != NULL; it = it->next) {
 		Itdb_Track *song;
 		char *filename;
+		const char *mount_path;
 
 		song = (Itdb_Track *)it->data;
- 		filename = ipod_path_to_uri (priv->ipod_mount_path,
- 					    song->ipod_path);
+		mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+ 		filename = ipod_path_to_uri (mount_path, song->ipod_path);
 		rb_static_playlist_source_add_location (RB_STATIC_PLAYLIST_SOURCE (playlist_source),
 							filename, -1);
 		g_free (filename);
 	}
 
-	priv->playlists = g_list_prepend (priv->playlists, playlist_source);
+	g_object_ref (G_OBJECT(playlist_source));
+	playlist->userdata = playlist_source;
+	playlist->userdata_destroy = g_object_unref;
+	playlist->userdata_duplicate = g_object_ref;
 
+	model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist_source));
+	g_signal_connect (model, "row-inserted",
+			          G_CALLBACK (playlist_track_added),
+			          playlist_source);
+	g_signal_connect (model, "entry-removed",
+			          G_CALLBACK (playlist_track_removed),
+			          playlist_source);
+
+	g_object_set_data (G_OBJECT (playlist_source),
+			           "ipod-source", source);
+	g_object_set_data (G_OBJECT (playlist_source),
+			           "itdb-playlist", playlist);
+	if (itdb_playlist_is_podcasts(playlist))
+			priv->podcast_pl = RB_STATIC_PLAYLIST_SOURCE (playlist_source);
 	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
 	g_object_unref (shell);
 }
@@ -344,7 +399,9 @@ load_ipod_playlists (RBiPodSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *it;
 
-	for (it = priv->ipod_db->playlists; it != NULL; it = it->next) {
+	for (it = rb_ipod_db_get_playlists (priv->ipod_db);
+	     it != NULL;
+	     it = it->next) {
 		Itdb_Playlist *playlist;
 
 		playlist = (Itdb_Playlist *)it->data;
@@ -361,6 +418,11 @@ load_ipod_playlists (RBiPodSource *source)
 }
 
 #ifdef ENABLE_IPOD_WRITING
+
+/* FIXME:  these should go away once we compile against new-enough libgpod */
+#define MEDIATYPE_AUDIO         0x0001
+#define MEDIATYPE_PODCAST       0x0004
+
 static Itdb_Track *
 create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 {
@@ -384,13 +446,42 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->time_played = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_LAST_PLAYED);
 	track->time_played = itdb_time_host_to_mac (track->time_played);
 	track->rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING);
+	track->rating *= ITDB_RATING_STEP;
 	track->app_rating = track->rating;
 	track->playcount = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_PLAY_COUNT);
 
+	if (rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
+		track->mediatype = MEDIATYPE_PODCAST;
+	} else {
+		track->mediatype = MEDIATYPE_AUDIO;
+	}
+
 	return track;
 }
 #endif
 
+static void add_offline_played_entry (RBiPodSource *source,
+				      RhythmDBEntry *entry,
+				      guint play_count)
+{
+	PlayedEntry *played_entry;
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+
+	if (play_count == 0) {
+		return;
+	}
+
+	if (priv->offline_plays == NULL) {
+		priv->offline_plays = g_queue_new();
+	}
+
+	played_entry = g_new0 (PlayedEntry, 1);
+	played_entry->entry = entry;
+	played_entry->play_count = play_count;
+
+	g_queue_push_tail (priv->offline_plays, played_entry);
+}
+
 static void
 add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 {
@@ -398,13 +489,13 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	RhythmDBEntryType entry_type;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	char *pc_path;
+	const char *mount_path;
 
 	/* Set URI */
 	g_object_get (source, "entry-type", &entry_type,
 		      NULL);
-
-	pc_path = ipod_path_to_uri (priv->ipod_mount_path,
-				    song->ipod_path);
+	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+	pc_path = ipod_path_to_uri (mount_path, song->ipod_path);
 	entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type,
 				    pc_path);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
@@ -510,7 +601,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	if (song->rating != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_DOUBLE);
-		g_value_set_double (&value, song->rating/20.0);
+		g_value_set_double (&value, song->rating/ITDB_RATING_STEP);
 		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_RATING,
 					       &value);
@@ -544,6 +635,11 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 
 	g_hash_table_insert (priv->entry_map, entry, song);
 
+	if (song->recent_playcount != 0) {
+		add_offline_played_entry (source, entry,
+					  song->recent_playcount);
+	}
+
 	rhythmdb_commit (RHYTHMDB (db));
 }
 
@@ -553,13 +649,194 @@ get_db_for_source (RBiPodSource *source)
 	RBShell *shell;
 	RhythmDB *db;
 
-  	g_object_get (source, "shell", &shell, NULL);
-  	g_object_get (shell, "db", &db, NULL);
-  	g_object_unref (shell);
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
+	g_object_unref (shell);
 
 	return db;
 }
 
+static gint
+compare_timestamps (gconstpointer a, gconstpointer b, gpointer data)
+{
+	PlayedEntry *lhs = (PlayedEntry *)a;
+	PlayedEntry *rhs = (PlayedEntry *)b;
+
+	gulong lhs_timestamp;
+	gulong rhs_timestamp;
+
+	lhs_timestamp =  rhythmdb_entry_get_ulong (lhs->entry,
+						   RHYTHMDB_PROP_LAST_PLAYED);
+
+	rhs_timestamp =  rhythmdb_entry_get_ulong (rhs->entry,
+						   RHYTHMDB_PROP_LAST_PLAYED);
+
+
+	return (int) (lhs_timestamp - rhs_timestamp);
+}
+
+static void
+remove_playcount_file (RBiPodSource *source)
+{
+        RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+        char *itunesdb_dir;
+        char *playcounts_file;
+        int result;
+	const char *mountpoint;
+	
+	mountpoint = rb_ipod_db_get_mount_path (priv->ipod_db);
+        itunesdb_dir = itdb_get_itunes_dir (mountpoint);
+        playcounts_file = itdb_get_path (itunesdb_dir, "Play Counts");
+        result = g_unlink (playcounts_file);
+        if (result == 0) {
+                rb_debug ("iPod Play Counts file successfully deleted");
+        } else {
+                rb_debug ("Failed to remove iPod Play Counts file: %s",
+                          strerror (errno));
+        }
+        g_free (itunesdb_dir);
+        g_free (playcounts_file);
+
+}
+
+static void
+send_offline_plays_notification (RBiPodSource *source)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	RhythmDB *db;
+	GValue val = {0, };
+
+	if (priv->offline_plays == NULL) {
+		return;
+	}
+
+	/* audioscrobbler expects data to arrive with increasing timestamps,
+	 * dunno if the sorting should be done in the audioscrobbler plugin,
+	 * or if this kind of "insider knowledge" is OK here
+	 */
+	g_queue_sort (priv->offline_plays,
+		      (GCompareDataFunc)compare_timestamps,
+		      NULL);
+
+	db = get_db_for_source (source);
+	g_value_init (&val, G_TYPE_ULONG);
+
+	while (!g_queue_is_empty (priv->offline_plays)) {
+		gulong last_play;
+		PlayedEntry *entry;
+		entry = (PlayedEntry*)g_queue_pop_head (priv->offline_plays);
+		last_play = rhythmdb_entry_get_ulong (entry->entry,
+						      RHYTHMDB_PROP_LAST_PLAYED);
+		g_value_set_ulong (&val, last_play);
+		rhythmdb_emit_entry_extra_metadata_notify (db, entry->entry,
+							   "rb:offlinePlay",
+							   &val);
+		g_free (entry);
+	}
+	g_value_unset (&val);
+	g_object_unref (G_OBJECT (db));
+
+	remove_playcount_file (source);
+}
+
+static void
+rb_ipod_source_entry_changed_cb (RhythmDB *db,
+				 RhythmDBEntry *entry,
+				 GSList *changes,
+				 RBiPodSource *source)
+{
+	GSList *t;
+
+	/* Ignore entries which are not iPod entries */
+	RhythmDBEntryType entry_type;
+	RhythmDBEntryType ipod_entry_type;
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+
+	entry_type = rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE);
+	g_object_get (G_OBJECT (source),
+		      "entry-type", &ipod_entry_type,
+		      NULL);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, ipod_entry_type);
+	/* ipod_entry_type can no longer be dereferenced, but its value 
+	 * hasn't been changed, so this comparison is valid
+	 */
+	if (entry_type != ipod_entry_type) {
+		return; 
+	};
+
+
+	/* If an interesting property was changed, update it on the iPod */
+	/* If the iPod database is being saved in a separate thread, this 
+	 * might not be 100% thread-safe, but at worse we'll modify a field
+	 * at the time it's being saved which will get a wrong value, but
+	 * that's the worse that can happen and that's pretty theoritical, 
+	 * I don't think avoiding it is worth the effort.
+	 */
+	for (t = changes; t; t = t->next) {
+		RhythmDBEntryChange *change = t->data;
+		switch (change->prop) {
+		case RHYTHMDB_PROP_RATING: {
+			Itdb_Track *track;
+			double old_rating;
+			double new_rating;
+
+			old_rating = g_value_get_double (&change->old);
+			new_rating = g_value_get_double (&change->new);
+			if (old_rating != new_rating) {
+				track = g_hash_table_lookup (priv->entry_map, 
+							     entry);
+				track->rating = new_rating * ITDB_RATING_STEP;
+				track->app_rating = track->rating;
+				rb_debug ("rating changed, saving db");
+				rb_ipod_db_save_async (priv->ipod_db);
+			} else {
+				rb_debug ("rating didn't change");
+			}
+			break;
+		}
+		case RHYTHMDB_PROP_PLAY_COUNT: {
+			Itdb_Track *track;
+			gulong old_playcount;
+			gulong new_playcount;
+
+			old_playcount = g_value_get_ulong (&change->old);
+			new_playcount = g_value_get_ulong (&change->new);
+			if (old_playcount != new_playcount) {
+				track = g_hash_table_lookup (priv->entry_map, 
+							     entry);
+				track->playcount = new_playcount;
+				rb_debug ("playcount changed, saving db");
+				rb_ipod_db_save_async (priv->ipod_db);
+			} else {
+				rb_debug ("playcount didn't change");
+			}
+			break;
+		}
+		case RHYTHMDB_PROP_LAST_PLAYED: {
+			Itdb_Track *track;
+			gulong old_lastplay;
+			gulong new_lastplay;
+
+			old_lastplay = g_value_get_ulong (&change->old);
+			new_lastplay = g_value_get_ulong (&change->new);
+			if (old_lastplay != new_lastplay) {
+				track = g_hash_table_lookup (priv->entry_map, 
+							     entry);
+				track->time_played = itdb_time_host_to_mac (new_lastplay);
+				rb_debug ("last play time changed, saving db");
+				rb_ipod_db_save_async (priv->ipod_db);
+			} else {
+				rb_debug ("last play time didn't change");
+			}
+			break;			
+		}
+		default:
+			rb_debug ("Ignoring property %d\n", change->prop);
+			break;
+		}
+	}
+}
+
 static gboolean
 load_ipod_db_idle_cb (RBiPodSource *source)
 {
@@ -571,12 +848,19 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 
 	db = get_db_for_source (source);
 
-  	g_assert (db != NULL);
- 	for (it = priv->ipod_db->tracks; it != NULL; it = it->next) {
+	g_assert (db != NULL);
+ 	for (it = rb_ipod_db_get_tracks (priv->ipod_db);
+	     it != NULL;
+	     it = it->next) {
 		add_ipod_song_to_db (source, db, (Itdb_Track *)it->data);
 	}
 
 	load_ipod_playlists (source);
+	send_offline_plays_notification (source);
+
+	g_signal_connect_object(G_OBJECT(db), "entry-changed", 
+				G_CALLBACK (rb_ipod_source_entry_changed_cb),
+				source, 0);
 
 	g_object_unref (db);
 
@@ -592,42 +876,22 @@ rb_ipod_load_songs (RBiPodSource *source)
 	GnomeVFSVolume *volume;
 
 	g_object_get (source, "volume", &volume, NULL);
-	priv->ipod_mount_path = rb_ipod_get_mount_path (volume);
-
- 	priv->ipod_db = itdb_parse (priv->ipod_mount_path, NULL);
+ 	priv->ipod_db = rb_ipod_db_new (volume);
 	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
-	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
-		Itdb_Playlist *mpl;
 
-		/* FIXME: we could set a different icon depending on the iPod
-		 * model
-		 */
-		mpl = itdb_playlist_mpl (priv->ipod_db);
-		if (mpl && mpl->name) {
+	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
+		const char *name;
+		name = rb_ipod_db_get_ipod_name (priv->ipod_db);
+		if (name) {
 			g_object_set (RB_SOURCE (source),
-				      "name", mpl->name,
+				      "name", name,
 				      NULL);
 		}
 		priv->load_idle_id = g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
 	}
-
 	g_object_unref (volume);
 }
 
-static gchar *
-rb_ipod_get_mount_path (GnomeVFSVolume *volume)
-{
-	gchar *path;
-	gchar *uri;
-
-	uri = gnome_vfs_volume_get_activation_uri (volume);
-	path = g_filename_from_uri (uri, NULL, NULL);
-	g_assert (path != NULL);
-	g_free (uri);
-
-	return path;
-}
-
 static gchar *
 rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
 {
@@ -645,13 +909,7 @@ rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
 		return NULL;
 	}
 
-#ifdef IPOD_SUPPORT
-	result = itdb_get_itunesdb_path (mount_point);
-#else
-	result = g_build_filename (mount_point,
-				   "iPod_Control/iTunes/iTunesDB",
-				   NULL);
-#endif
+	result = itdb_get_itunes_dir(mount_point);
 
 	g_free (mount_point);
 	return result;
@@ -701,7 +959,7 @@ rb_ipod_is_volume_ipod (GnomeVFSVolume *volume)
 	return rb_ipod_volume_has_ipod_db (volume);
 }
 
-#ifdef HAVE_HAL_0_5
+#ifdef HAVE_HAL
 
 static gboolean
 hal_udi_is_ipod (const char *udi)
@@ -745,48 +1003,6 @@ hal_udi_is_ipod (const char *udi)
 
 	parent_name = libhal_device_get_property_string (ctx, parent_udi,
 			"storage.model", &error);
-#ifdef IPOD_SUPPORT
-	{
-		char *spider_udi;
-		int vnd_id = 0;
-		int product_id = 0;
-
-		spider_udi = g_strdup(parent_udi);
-		while (vnd_id == 0 && product_id == 0 && spider_udi != NULL) {
-			char *old_udi = spider_udi;
-			spider_udi =  libhal_device_get_property_string (ctx, spider_udi,
-					"info.parent", &error);
-			if (dbus_error_is_set (&error)) {
-				dbus_error_free (&error);
-				dbus_error_init (&error);
-				spider_udi = NULL;
-				break;
-			}
-			g_free(old_udi);
-
-			vnd_id = libhal_device_get_property_int (ctx, spider_udi,
-				"usb.vendor_id", &error);
-			if (dbus_error_is_set(&error)) {
-				dbus_error_free (&error);
-				dbus_error_init (&error);
-				vnd_id = 0;
-			}
-
-			product_id = libhal_device_get_property_int (ctx, spider_udi,
-				"usb.product_id", &error);
-			if (dbus_error_is_set(&error)) {
-				dbus_error_free (&error);
-				dbus_error_init (&error);
-				product_id = 0;
-			}
-		}
-		g_free (spider_udi);
-
-		if (vnd_id == PHONE_VENDOR_ID && product_id == PHONE_PRODUCT_ID) {
-			result = TRUE;
-		}
-	}
-#endif
 	if (parent_name == NULL || dbus_error_is_set (&error))
 		goto end;
 
@@ -813,41 +1029,6 @@ end:
 
 	return result;
 }
-
-#elif HAVE_HAL_0_2
-
-static gboolean
-hal_udi_is_ipod (const char *udi)
-{
-	LibHalContext *ctx;
-	char *parent_udi;
-	char *parent_name;
-	gboolean result;
-
-	result = FALSE;
-	ctx = hal_initialize (NULL, FALSE);
-	if (ctx == NULL) {
-		/* FIXME: should we return an error somehow so that we can
-		 * fall back to a check for iTunesDB presence instead ?
-		 */
-		return FALSE;
-	}
-	parent_udi = hal_device_get_property_string (ctx, udi,
-			"info.parent");
-	parent_name = hal_device_get_property_string (ctx, parent_udi,
-			"storage.model");
-	g_free (parent_udi);
-
-	if (parent_name != NULL && strcmp (parent_name, "iPod") == 0) {
-		result = TRUE;
-	}
-
-	g_free (parent_name);
-	hal_shutdown (ctx);
-
-	return result;
-}
-
 #endif
 
 static GList*
@@ -867,17 +1048,6 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-static void
-remove_track_from_db (Itdb_Track *track)
-{
-	GList *it;
-
-	for (it = track->itdb->playlists; it != NULL; it = it->next) {
-		itdb_playlist_remove_track ((Itdb_Playlist *)it->data, track);
-	}
-	itdb_track_remove (track);
-}
-
 static void
 impl_move_to_trash (RBSource *asource)
 {
@@ -905,40 +1075,17 @@ impl_move_to_trash (RBSource *asource)
 			continue;
 		}
 
- 		remove_track_from_db (track);
+		rb_ipod_db_remove_track (priv->ipod_db, track);
 		g_hash_table_remove (priv->entry_map, entry);
 		rhythmdb_entry_move_to_trash (db, entry);
 		rhythmdb_commit (db);
 	}
 
-	if (sel != NULL) {
-                itdb_schedule_save (RB_IPOD_SOURCE (asource));
-	}
-
-  	g_object_unref (db);
+	g_object_unref (db);
 
 	g_list_free (sel);
 }
 
-static void
-itdb_schedule_save (RBiPodSource *source)
-{
-	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
-
-	/* FIXME: should probably be delayed a bit to avoid doing
-	 * it after each file when we are copying several files
-	 * consecutively
-	 * FIXME: or this function could be called itdb_set_dirty, and we'd
-	 * have a timeout firing every 5 seconds and saving the db if it's
-	 * dirty
-	 */
-	itdb_write (priv->ipod_db, NULL);
-
-	if (priv->needs_shuffle_db) {
-       		itdb_shuffle_write (priv->ipod_db, NULL);
-	}
-}
-
 #ifdef ENABLE_IPOD_WRITING
 static char *
 impl_build_dest_uri (RBRemovableMediaSource *source,
@@ -949,9 +1096,12 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	const char *uri;
 	char *dest;
+	const char *mount_path;
 
 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-	dest = ipod_get_filename_for_uri (priv->ipod_mount_path,  uri, mimetype, extension);
+	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+	dest = ipod_get_filename_for_uri (mount_path,  uri, 
+					  mimetype, extension);
 	if (dest != NULL) {
 		char *dest_uri;
 
@@ -963,6 +1113,76 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 	return NULL;
 }
 
+static void
+artwork_notify_cb (RhythmDB *db,
+		   RhythmDBEntry *entry,
+		   const gchar *property_name,
+		   const GValue *metadata,
+		   RBiPodSource *isource)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
+	Itdb_Track *song;
+	GdkPixbuf *pixbuf;
+
+	g_return_if_fail (G_VALUE_HOLDS (metadata, GDK_TYPE_PIXBUF));
+	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
+		
+	song = g_hash_table_lookup (priv->artwork_request_map, entry);
+	if (song == NULL)
+		return;
+
+	rb_ipod_db_set_thumbnail (priv->ipod_db, song, pixbuf);
+	g_hash_table_remove (priv->artwork_request_map, entry);
+}
+
+static void
+request_artwork (RBiPodSource *isource,
+		 RhythmDBEntry *entry,
+		 RhythmDB *db,
+		 Itdb_Track *song)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
+	GValue *metadata;
+
+	if (priv->artwork_request_map == NULL) {
+		priv->artwork_request_map = g_hash_table_new (g_direct_hash, g_direct_equal);
+	}
+
+	g_hash_table_insert (priv->artwork_request_map, entry, song);
+
+	if (priv->artwork_notify_id == 0) {
+		priv->artwork_notify_id = g_signal_connect_object (db, "entry-extra-metadata-notify::rb:coverArt",
+								   (GCallback)artwork_notify_cb, isource, 0);
+	}
+
+	metadata = rhythmdb_entry_request_extra_metadata (db, entry, "rb:coverArt");
+	if (metadata) {
+		artwork_notify_cb (db, entry, "rb:coverArt", metadata, isource);
+	}
+}
+
+static void
+add_to_podcasts (RBiPodSource *source, Itdb_Track *song)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	gchar *filename;
+	const gchar *mount_path;
+
+	if (priv->podcast_pl == NULL) {
+		/* No Podcast playlist on the iPod, create a new one */
+		Itdb_Playlist *ipod_playlist;
+		ipod_playlist = itdb_playlist_new (_("Podcasts"), FALSE);
+		itdb_playlist_set_podcasts (ipod_playlist);
+		rb_ipod_db_add_playlist (priv->ipod_db, ipod_playlist);
+		add_rb_playlist (source, ipod_playlist);
+	}
+
+	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+  	filename = ipod_path_to_uri (mount_path, song->ipod_path);
+ 	rb_static_playlist_source_add_location (priv->podcast_pl, filename, -1);
+	g_free (filename);
+}
+
 static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
@@ -979,16 +1199,31 @@ impl_track_added (RBRemovableMediaSource *source,
 	if (song != NULL) {
 		RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 		char *filename;
+		const char *mount_path;
 
 		filename = g_filename_from_uri (dest, NULL, NULL);
-		song->ipod_path = ipod_path_from_unix_path (priv->ipod_mount_path, filename);
+		mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+		song->ipod_path = ipod_path_from_unix_path (mount_path,
+							    filename);
 		g_free (filename);
-		itdb_track_add (priv->ipod_db, song, -1);
-		itdb_playlist_add_track (itdb_playlist_mpl (priv->ipod_db),
-					 song, -1);
 
+		if (song->mediatype == MEDIATYPE_PODCAST) {
+			add_to_podcasts (isource, song);
+		}
+#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
+		/* reuse that #define since both functions were added to 
+		 * libgpod CVS HEAD around the same time
+		 */
+		Itdb_Device *device;
+		device = rb_ipod_db_get_device (priv->ipod_db);		
+		if (device && itdb_device_supports_artwork (device)) {
+			request_artwork (isource, entry, db, song);
+		}
+#else 
+		request_artwork (isource, entry, db, song);
+#endif
 		add_ipod_song_to_db (isource, db, song);
-		itdb_schedule_save (isource);
+		rb_ipod_db_add_track (priv->ipod_db, song);
 	}
 
 	g_object_unref (db);
@@ -1232,15 +1467,32 @@ impl_delete_thyself (RBSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *p;
 
-	for (p = priv->playlists; p != NULL; p = p->next) {
-		RBSource *playlist = RB_SOURCE (p->data);
-		rb_source_delete_thyself (playlist);
-	}
-	g_list_free (priv->playlists);
-	priv->playlists = NULL;
+	for (p = rb_ipod_db_get_playlists (priv->ipod_db);
+	     p != NULL;
+	     p = p->next) {
+		Itdb_Playlist *playlist = (Itdb_Playlist *)p->data;
+		if (!itdb_playlist_is_mpl (playlist) && !playlist->is_spl) {
+			RBSource *rb_playlist;
+			RhythmDBQueryModel *model;
 
-	itdb_free (priv->ipod_db);
-	priv->ipod_db = NULL;
+			rb_playlist = RB_SOURCE (playlist->userdata);
+			model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (rb_playlist));
+
+			/* remove these to ensure they aren't called during source deletion */
+			g_signal_handlers_disconnect_by_func (model,
+							      G_CALLBACK (playlist_track_added),
+							      rb_playlist);
+			g_signal_handlers_disconnect_by_func (model,
+							      G_CALLBACK (playlist_track_removed),
+							      rb_playlist);
+			rb_source_delete_thyself (rb_playlist);
+		}
+	}
+
+	if (priv->ipod_db != NULL) {
+		g_object_unref (G_OBJECT (priv->ipod_db));
+		priv->ipod_db = NULL;
+	}
 
 	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
 }
