@@ -50,6 +50,11 @@
 #include "rb-player.h"
 #include "rb-metadata.h"
 #include "rb-plugin.h"
+#include "rb-cut-and-paste-code.h"
+
+/* icon names */
+#define IRADIO_SOURCE_ICON  "library-internet-radio"
+#define IRADIO_NEW_STATION_ICON "internet-radio-new"
 
 static void rb_iradio_source_class_init (RBIRadioSourceClass *klass);
 static void rb_iradio_source_init (RBIRadioSource *source);
@@ -159,7 +164,7 @@ struct RBIRadioSourcePrivate
 
 static GtkActionEntry rb_iradio_source_actions [] =
 {
-	{ "MusicNewInternetRadioStation", GTK_STOCK_NEW, N_("New Internet _Radio Station"), "<control>I",
+	{ "MusicNewInternetRadioStation", IRADIO_NEW_STATION_ICON, N_("New Internet _Radio Station..."), "<control>I",
 	  N_("Create a new Internet Radio station"),
 	  G_CALLBACK (rb_iradio_source_cmd_new_station) }
 };
@@ -214,9 +219,9 @@ rb_iradio_source_init (RBIRadioSource *source)
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   "stock_channel",
+					   IRADIO_SOURCE_ICON,
 					   size,
 					   0, NULL);
 	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
@@ -422,6 +427,7 @@ rb_iradio_source_new (RBShell *shell, RBPlugin *plugin)
 					  "name", _("Radio"),
 					  "shell", shell,
 					  "entry-type", entry_type,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  "plugin", plugin,
 					  NULL));
 	rb_shell_register_entry_type_for_source (shell, source, entry_type);
@@ -457,6 +463,8 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	RhythmDBEntry *entry;
 	GValue val = { 0, };
 	char *real_uri = NULL;
+	char *fixed_title;
+	char *fixed_genre = NULL;
 	RhythmDBEntryType entry_type;
 
 	real_uri = guess_uri_scheme (uri);
@@ -479,20 +487,27 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	}
 
 	g_value_init (&val, G_TYPE_STRING);
-	if (title)
-		g_value_set_static_string (&val, title);
-	else
-		g_value_take_string (&val,
-				     gnome_vfs_format_uri_for_display (uri));
+	if (title) {
+		fixed_title = rb_make_valid_utf8 (title, '?');
+	} else {
+		fixed_title = gnome_vfs_format_uri_for_display (uri);
+	}
+	g_value_take_string (&val, fixed_title);
 
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
 	g_value_reset (&val);
 
-	if ((!genre) || (strcmp (genre, "") == 0))
+	if ((!genre) || (strcmp (genre, "") == 0)) {
 		genre = _("Unknown");
+	} else {
+		fixed_genre = rb_make_valid_utf8 (genre, '?');
+		genre = fixed_genre;
+	}
+
 	g_value_set_string (&val, genre);
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	g_value_unset (&val);
+	g_free (fixed_genre);
 
 	g_value_init (&val, G_TYPE_DOUBLE);
 	g_value_set_double (&val, 0.0);
@@ -835,13 +850,27 @@ rb_iradio_source_do_query (RBIRadioSource *source)
 	source->priv->setting_new_query = FALSE;
 }
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+static void
+handle_playlist_entry_cb (TotemPlParser *playlist,
+			  const char *uri,
+			  GHashTable *metadata,
+			  RBIRadioSource *source)
+#else
 static void
 handle_playlist_entry_cb (TotemPlParser *playlist,
 			  const char *uri,
 			  const char *title,
 			  const char *genre,
 			  RBIRadioSource *source)
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 {
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+	const char *title, *genre;
+
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+	genre = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 	rb_iradio_source_add_station (source, uri, title, genre);
 }
 
@@ -856,9 +885,15 @@ rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 	if (real_uri)
 		uri = real_uri;
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+	g_signal_connect_object (parser, "entry-parsed",
+				 G_CALLBACK (handle_playlist_entry_cb),
+				 source, 0);
+#else
 	g_signal_connect_object (parser, "entry",
 				 G_CALLBACK (handle_playlist_entry_cb),
 				 source, 0);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 	if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
 		g_object_set (parser, "recurse", FALSE, NULL);
 
@@ -1019,6 +1054,7 @@ check_entry_type (RBIRadioSource *source, RhythmDBEntry *entry)
 
 static void
 info_available_cb (RBPlayer *backend,
+		   const char *uri,
 		   RBMetaDataField field,
 		   GValue *value,
 		   RBIRadioSource *source)
