@@ -1,3 +1,22 @@
+# -*- Mode: python; coding: utf-8; tab-width: 8; indent-tabs-mode: t; -*-
+#
+# Copyright (C) 2006 Adam Zimmerman  <adam_zimmerman@sfu.ca>
+# Copyright (C) 2006 James Livingston  <doclivingston@gmail.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
+
 import rb, rhythmdb
 from TrackListHandler import TrackListHandler
 from BuyAlbumHandler import BuyAlbumHandler, MagnatunePurchaseError
@@ -9,6 +28,7 @@ import xml
 import urllib, zipfile
 
 has_gnome_keyring = False
+
 #try:
 #	import gnomekeyring
 #	has_gnome_keyring = True
@@ -23,7 +43,7 @@ magnatune_dir = gnome.user_dir_get() + "rhythmbox/magnatune/"
 magnatune_song_info_uri = gnomevfs.URI("http://magnatune.com/info/song_info_xml.zip")
 local_song_info_uri = gnomevfs.URI(magnatune_dir + "song_info.xml")
 local_song_info_temp_uri = gnomevfs.URI(magnatune_dir + "song_info.xml.zip.tmp")
-
+ALBUM_ART_URL = 'http://www.magnatune.com/music/%s/%s/cover.jpg'
 
 class MagnatuneSource(rb.BrowserSource):
 	__gproperties__ = {
@@ -36,11 +56,13 @@ class MagnatuneSource(rb.BrowserSource):
 	def __init__(self):
 
 		rb.BrowserSource.__init__(self, name=_("Magnatune"))
+		self.__db = None
 
 		# track data
 		self.__sku_dict = {}
 		self.__home_dict = {}
 		self.__buy_dict = {}
+		self.__art_dict = {}
 
 		# catalogue stuff
 		self.__activated = False
@@ -72,7 +94,6 @@ class MagnatuneSource(rb.BrowserSource):
 		else:
 			raise AttributeError, 'unknown property %s' % property.name
 
-
 	#
 	# RBSource methods
 	#
@@ -140,39 +161,51 @@ class MagnatuneSource(rb.BrowserSource):
 			self.__notify_id = 0
 
 		if self.__xfer_handle is not None:
-			self.__xfer_handle.close(lambda handle, exc: None) #FIXME: report it?
+			self.__xfer_handle.cancel()
 			self.__xfer_handle = None
 
 		self.__client.set_string("/apps/rhythmbox/plugins/magnatune/sorting", self.get_entry_view().get_sorting_type())
 
 		rb.BrowserSource.do_impl_delete_thyself (self)
 
-
 	#
 	# methods for use by plugin and UI
 	#
 
-
 	def display_artist_info(self):
 		tracks = self.get_entry_view().get_selected_entries()
 		urls = set([])
 
 		for tr in tracks:
-			url = self.__home_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			sku = self.__sku_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			url = self.__home_dict[sku]
 			if url not in urls:
 				gnomevfs.url_show(url)
 				urls.add(url)
-	
+
 	def buy_cd(self):
 		tracks = self.get_entry_view().get_selected_entries()
 		urls = set([])
 
 		for tr in tracks:
-			url = self.__buy_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			sku = self.__sku_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			url = self.__buy_dict[sku]
 			if url not in urls:
 				gnomevfs.url_show(url)
 				urls.add(url)
 
+	def radio_toggled(self, gladexml):
+		gc = gladexml.get_widget("radio_gc").get_active()
+		gladexml.get_widget("remember_cc_details").set_sensitive(not gc)
+		gladexml.get_widget("name_entry").set_sensitive(not gc)
+		gladexml.get_widget("cc_entry").set_sensitive(not gc)
+		gladexml.get_widget("mm_entry").set_sensitive(not gc)
+		gladexml.get_widget("yy_entry").set_sensitive(not gc)
+		
+		gladexml.get_widget("gc_entry").set_sensitive(gc)
+		if not gc:
+			gladexml.get_widget("gc_entry").set_text("")
+	
 	def purchase_album(self):
 		try:
 			library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0] # Just use the first library location
@@ -193,7 +226,10 @@ class MagnatuneSource(rb.BrowserSource):
 			album = self.__db.entry_get(track, rhythmdb.PROP_ALBUM)
 
 			gladexml = gtk.glade.XML(self.__plugin.find_file("magnatune-purchase.glade"))
-
+			cb_dict = {"rb_magnatune_on_radio_cc_toggled_cb":lambda w:self.radio_toggled(gladexml)}
+			gladexml.signal_autoconnect(cb_dict)
+			
+			gladexml.get_widget("gc_entry").set_sensitive(False)
 			gladexml.get_widget("pay_combobox").set_active(self.__client.get_int(self.__plugin.gconf_keys['pay']) - 5)
 			gladexml.get_widget("audio_combobox").set_active(self.__plugin.format_list.index(self.__client.get_string(self.__plugin.gconf_keys['format'])))
 			gladexml.get_widget("info_label").set_markup(_("Would you like to purchase the album <i>%(album)s</i> by '%(artist)s'?") % {"album":album, "artist":artist})
@@ -219,7 +255,6 @@ class MagnatuneSource(rb.BrowserSource):
 
 				gladexml.get_widget("remember_cc_details").set_active(False)
 
-
 			window = gladexml.get_widget("purchase_dialog")
 			if window.run() == gtk.RESPONSE_ACCEPT:
 				amount = gladexml.get_widget("pay_combobox").get_active() + 5
@@ -229,13 +264,15 @@ class MagnatuneSource(rb.BrowserSource):
 				ccmonth = str(gladexml.get_widget("mm_entry").get_active() + 1).zfill(2)
 				name = gladexml.get_widget("name_entry").get_text()
 				email = gladexml.get_widget("email_entry").get_text()
+				gc = gladexml.get_widget("radio_gc").get_active()
+				gc_text = gladexml.get_widget("gc_entry").get_text()
 
 				if gladexml.get_widget("remember_cc_details").props.active:
 					self.__plugin.store_cc_details(ccnumber, ccyear, ccmonth, name, email)
 				else:
 					self.__plugin.clear_cc_details()
 
-				self.__purchase_album (sku, amount, format, ccnumber, ccyear, ccmonth, name, email)
+				self.__buy_album (sku, amount, format, ccnumber, ccyear, ccmonth, name, email, gc, gc_text)
 
 			window.destroy()
 
@@ -292,13 +329,18 @@ class MagnatuneSource(rb.BrowserSource):
 				return
 
 		parser = xml.sax.make_parser()
-		parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__buy_dict))
+		parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__buy_dict, self.__art_dict))
 		handle.read (64 * 1024, self.__load_catalogue_read_cb, parser)
 
 	def __load_catalogue(self):
 		self.__notify_status_changed()
 		self.__load_handle = gnomevfs.async.open (local_song_info_uri, self.__load_catalogue_open_cb)
 
+	def __find_song_info(self, catalogue):
+		for info in catalogue.infolist():
+			if info.filename.endswith("song_info.xml"):
+				return info.filename;
+		return None
 
 	def __download_update_cb (self, _reserved, info, moving):
 		self.__load_current_size = info.bytes_copied
@@ -309,12 +351,17 @@ class MagnatuneSource(rb.BrowserSource):
 			# done downloading, unzip to real location
 			catalog = zipfile.ZipFile(local_song_info_temp_uri.path)
 			out = create_if_needed(local_song_info_uri, gnomevfs.OPEN_WRITE)
-			out.write(catalog.read("opt/magnatune/info/song_info.xml"))
+			filename = self.__find_song_info(catalog)
+			if filename is None:
+				rb.error_dialog(title="Unable to load catalogue", message=_("Rhythmbox could not understand the Magnatune catalogue, please file a bug."))
+				return
+			out.write(catalog.read(filename))
 			out.close()
 			catalog.close()
 			gnomevfs.unlink(local_song_info_temp_uri)
 			self.__updating = False
 			self.__load_catalogue()
+			self.__xfer_handle = None
 		else:
 			#print info
 			pass
@@ -369,8 +416,8 @@ class MagnatuneSource(rb.BrowserSource):
 	def __show_loading_screen(self, show):
 		if self.__info_screen is None:
 			# load the glade stuff
-			gladexml = gtk.glade.XML(self.__plugin.find_file("magnatune-loading.glade"), root="magnatune_loading_vbox")
-			self.__info_screen = gladexml.get_widget("magnatune_loading_vbox")
+			gladexml = gtk.glade.XML(self.__plugin.find_file("magnatune-loading.glade"), root="magnatune_loading_scrolledwindow")
+			self.__info_screen = gladexml.get_widget("magnatune_loading_scrolledwindow")
 			self.pack_start(self.__info_screen)
 			self.get_entry_view().set_no_show_all (True)
 			self.__info_screen.set_no_show_all (True)
@@ -387,36 +434,26 @@ class MagnatuneSource(rb.BrowserSource):
 		if self.__notify_id == 0:
 			self.__notify_id = gobject.idle_add(change_idle_cb)
 
-
 	#
 	# internal purchasing code
 	#
-	def __purchase_album(self, sku, pay, format, ccnumber, ccyear, ccmonth, name, email):
+	def __buy_album(self, sku, pay, format, ccnumber, ccyear, ccmonth, name, email, gc, gc_text): # http://magnatune.com/info/api#purchase
 		print "purchasing tracks:", sku, pay, format, name, email
-
-		try:
-			self.__buy_album(sku, pay, format, name, email, ccnumber, ccyear, ccmonth)
-		except MagnatunePurchaseError, e:
-			error_dlg = gtk.Dialog(title="Error", flags=gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK))
-			label = gtk.Label(_("An error occurred while trying to purchase the album.\nThe Magnatune server returned:\n%s") % str(e))
-			error_dlg.vbox.pack_start(label)
-			label.set_selectable(True)
-			label.show()
-			error_dlg.connect("response", lambda w, r: w.destroy())
-			error_dlg.show()
-
-	def __buy_album(self, sku, pay, format, name, email, ccnumber, ccyear, ccmonth): # http://magnatune.com/info/api#purchase
-		url = "https://magnatune.com/buy/buy_dl_cc_xml?"
-		url = url + urllib.urlencode({
+		url_dict = {
 						'id':	magnatune_partner_id,
 						'sku':	sku,
 						'amount': pay,
-						'cc':	ccnumber,
-						'yy':	ccyear,
-						'mm':	ccmonth,
 						'name': name,
 						'email':email
-					})
+			}
+		if gc:
+			url_dict['gc'] = gc
+		else:
+			url_dict['cc'] = ccnumber
+			url_dict['yy'] = ccyear
+			url_dict['mm'] = ccmonth
+		url = "https://magnatune.com/buy/buy_dl_cc_xml?"
+		url = url + urllib.urlencode(url_dict)
 
 		buy_album_handler = BuyAlbumHandler(format) # so we can get the url and auth info
 		auth_parser = xml.sax.make_parser()
@@ -469,9 +506,14 @@ class MagnatuneSource(rb.BrowserSource):
 				print data
 				auth_parser.feed(data)
 				handle.read(64 * 1024, self.__auth_read_cb, parser)
+			except MagnatunePurchaseError, e:
+				self.__wait_dlg.destroy()
+				rb.error_dialog(title = _("Purchase Error"),
+						message = _("An error occurred while trying to purchase the album.\nThe Magnatune server returned:\n%s") % str(e))
 			except Exception, e:
-				print e
-				raise e
+				self.__wait_dlg.destroy()
+				rb.error_dialog(title = _("Error"),
+						message = _("An error occurred while trying to purchase the album.\nThe error text is:\n%s") % str(e))
 
 	def __download_album(self, audio_dl_uri):
 			library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0] # Just use the first library location
@@ -551,6 +593,21 @@ class MagnatuneSource(rb.BrowserSource):
 		manager = shell.get_player().get_property('ui-manager')
 		manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(False)
 
+	def playing_entry_changed (self, entry):
+		if not self.__db or not entry:
+			return
+
+		if entry.get_entry_type() != self.__db.entry_type_get_by_name("MagnatuneEntryType"):
+			return
+
+		gobject.idle_add (self.emit_cover_art_uri, entry)
+
+	def emit_cover_art_uri (self, entry):
+		sku = self.__sku_dict[self.__db.entry_get(entry, rhythmdb.PROP_LOCATION)]
+		url = self.__art_dict[sku]
+		self.__db.emit_entry_extra_metadata_notify (entry, 'rb:coverArt-uri', url)
+		return False
+
 gobject.type_register(MagnatuneSource)
 
 
@@ -564,8 +621,6 @@ def create_if_needed(uri, mode):
 		out = gnomevfs.open(uri, open_mode=mode)
 	return out
 
-
-
 class URIIterator:
 	def __init__(self, uri):
 		self.uri_list = uri.dirname.split("/")[1:] # dirname starts with /
