@@ -16,10 +16,12 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
+#include "config.h"
+
 #define _XOPEN_SOURCE
 #define __EXTENSIONS__  /* get strptime */
 #include <string.h>
@@ -33,7 +35,7 @@
 #include <gtk/gtk.h>
 
 #include "rb-debug.h"
-#include "rb-podcast-parse.h" 
+#include "rb-podcast-parse.h"
 
 #define BUFFER_SIZE 256
 
@@ -44,7 +46,7 @@ struct RBPoadcastLoadContext
 	GString *prop_value;
 	RBPodcastChannel *channel_data;
 	RBPodcastItem *item_data;
-    
+
 	enum {
 		RB_PODCAST_PARSER_STATE_START,
 		RB_PODCAST_PARSER_STATE_RSS,
@@ -90,14 +92,14 @@ rb_set_channel_value (struct RBPoadcastLoadContext *ctx,
 
 	dvalue = xmlCharStrdup (value);
 	g_strstrip ((char *)dvalue);
-   
+
 	if (!strcmp (name, "title")) {
 		ctx->channel_data->title = dvalue;
-	} else if (!strcmp (name, "language")) { 
+	} else if (!strcmp (name, "language")) {
 		ctx->channel_data->lang = dvalue;
 	} else if (!strcmp (name, "itunes:subtitle")) {
 		ctx->channel_data->subtitle = dvalue;
-	} else if (!strcmp (name, "itunes:summary")) { 
+	} else if (!strcmp (name, "itunes:summary")) {
 		ctx->channel_data->summary = dvalue;
 	} else if (!strcmp (name, "description")) {
 		ctx->channel_data->description = dvalue;
@@ -121,7 +123,6 @@ rb_set_channel_value (struct RBPoadcastLoadContext *ctx,
 	}
 }
 
-
 static void
 rb_set_item_value (struct RBPoadcastLoadContext *ctx,
 		   const char *name,
@@ -153,7 +154,6 @@ rb_set_item_value (struct RBPoadcastLoadContext *ctx,
 	}
 }
 
-
 static void
 rb_insert_item (struct RBPoadcastLoadContext *ctx)
 {
@@ -176,11 +176,11 @@ rb_validate_channel_propert (const char *name)
 		return FALSE;
 	}
 
-	if (!strcmp (name, "title") || 
+	if (!strcmp (name, "title") ||
 	    !strcmp (name, "language") ||
 	    !strcmp (name, "itunes:subtitle") ||
 	    !strcmp (name, "itunes:summary") ||
-	    !strcmp (name, "description") || 
+	    !strcmp (name, "description") ||
 	    !strcmp (name, "generator") ||
 	    !strcmp (name, "itunes:author") ||
 	    !strcmp (name, "webMaster") ||
@@ -191,7 +191,7 @@ rb_validate_channel_propert (const char *name)
 	} else {
 		return FALSE;
 	}
-    
+
 }
 
 static gboolean
@@ -207,15 +207,14 @@ rb_validate_item_propert (const char *name)
 	    !strcmp (name, "description") ||
 	    !strcmp (name, "author") ||
 	    !strcmp (name, "itunes:duration") ) {
-	    
+
 		return TRUE;
 	} else {
 		return FALSE;
 	}
 }
 
-
-static void 
+static void
 rb_podcast_parser_start_element (struct RBPoadcastLoadContext *ctx,
 				 const char *name,
 				 const char **attrs)
@@ -234,7 +233,7 @@ rb_podcast_parser_start_element (struct RBPoadcastLoadContext *ctx,
 
 			break;
 		}
-        
+
         case RB_PODCAST_PARSER_STATE_RSS:
 		{
 			if (!strcmp (name, "channel")) {
@@ -248,10 +247,19 @@ rb_podcast_parser_start_element (struct RBPoadcastLoadContext *ctx,
 
         case RB_PODCAST_PARSER_STATE_CHANNEL:
 		{
- 
-			if (strcmp (name, "image") == 0
-			    || strcmp (name, "itunes:image") == 0) {
+			if (strcmp (name, "image") == 0)
+			{
 				ctx->state = RB_PODCAST_PARSER_STATE_IMG;
+			} else if (strcmp (name, "itunes:image") == 0) {
+				for (; attrs && *attrs; attrs +=2) {
+					if (!strcmp (*attrs, "href")) {
+						const char *href_value = *(attrs + 1);
+						rb_set_channel_value (ctx, "img", href_value);
+					}
+				}
+
+				ctx->state = RB_PODCAST_PARSER_STATE_IMG;
+
 			} else if (!strcmp (name, "item")) {
 				ctx->item_data = rb_podcast_initializa_item ();
 				ctx->state = RB_PODCAST_PARSER_STATE_ITEM;
@@ -303,6 +311,10 @@ rb_podcast_parser_start_element (struct RBPoadcastLoadContext *ctx,
         case RB_PODCAST_PARSER_STATE_CHANNEL_PROPERTY:
         case RB_PODCAST_PARSER_STATE_ITEM_PROPERTY:
         case RB_PODCAST_PARSER_STATE_IMG_PROPERTY:
+		rb_debug ("nested element inside property; treating as unknown");
+		ctx->in_unknown_elt++;
+		break;
+
         case RB_PODCAST_PARSER_STATE_END:
 		break;
 	default:
@@ -311,9 +323,8 @@ rb_podcast_parser_start_element (struct RBPoadcastLoadContext *ctx,
 	}
 }
 
-
 static void
-rb_podcast_parser_end_element (struct RBPoadcastLoadContext *ctx, 
+rb_podcast_parser_end_element (struct RBPoadcastLoadContext *ctx,
 			       const char *name)
 {
 	rb_debug ("End element: %s state: %d", name, ctx->state);
@@ -323,12 +334,12 @@ rb_podcast_parser_end_element (struct RBPoadcastLoadContext *ctx,
 		rb_debug ("Unknown element");
 		return;
 	}
-               
+
 	switch (ctx->state) {
         case RB_PODCAST_PARSER_STATE_START:
 		ctx->state = RB_PODCAST_PARSER_STATE_END;
 		break;
-            
+
         case RB_PODCAST_PARSER_STATE_RSS:
 		ctx->state = RB_PODCAST_PARSER_STATE_START;
 		break;
@@ -336,7 +347,7 @@ rb_podcast_parser_end_element (struct RBPoadcastLoadContext *ctx,
         case RB_PODCAST_PARSER_STATE_CHANNEL:
 		ctx->state = RB_PODCAST_PARSER_STATE_RSS;
 		break;
-            
+
         case RB_PODCAST_PARSER_STATE_CHANNEL_PROPERTY:
 		{
 			rb_set_channel_value (ctx, name, ctx->prop_value->str);
@@ -380,7 +391,6 @@ rb_podcast_parser_end_element (struct RBPoadcastLoadContext *ctx,
 		break;
 	}
 }
-    
 
 static void
 rb_podcast_parser_characters (struct RBPoadcastLoadContext *ctx,
@@ -390,7 +400,7 @@ rb_podcast_parser_characters (struct RBPoadcastLoadContext *ctx,
 	switch (ctx->state) {
         case RB_PODCAST_PARSER_STATE_CHANNEL_PROPERTY:
         case RB_PODCAST_PARSER_STATE_ITEM_PROPERTY:
-        case RB_PODCAST_PARSER_STATE_IMG_PROPERTY:            
+        case RB_PODCAST_PARSER_STATE_IMG_PROPERTY:
 		g_string_append_len (ctx->prop_value, data, len);
            	break;
         case RB_PODCAST_PARSER_STATE_START:
@@ -406,31 +416,44 @@ rb_podcast_parser_characters (struct RBPoadcastLoadContext *ctx,
 	}
 }
 
-
 gboolean
 rb_podcast_parse_load_feed (RBPodcastChannel *data,
 			    const char *file_name)
 {
-	xmlParserCtxtPtr ctxt;
+	xmlParserCtxtPtr parser;
 	xmlSAXHandlerPtr sax_handler = NULL;
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
 	gint file_size;
 	gchar *buffer = NULL;
-    
+	const char *query_string;
+
 	struct RBPoadcastLoadContext *ctx = NULL;
 
 	data->url = xmlCharStrdup (file_name);
 
-	if (!g_str_has_suffix (file_name, ".rss") && !g_str_has_suffix (file_name, ".xml")) {
+	/* if the URL has a .rss or .xml extension (before the query string),
+	 * don't bother checking the MIME type.
+	 */
+	query_string = strchr (file_name, '?');
+	if (query_string == NULL) {
+		query_string = file_name + strlen (file_name);
+	}
+
+	if (strncmp (query_string - 4, ".rss", 4) == 0 ||
+	    strncmp (query_string - 4, ".xml", 4) == 0) {
+		rb_debug ("not checking mime type for %s", file_name);
+	} else {
 		gboolean invalid_mime_type;
 
+		rb_debug ("checking mime type for %s", file_name);
 		info = gnome_vfs_file_info_new ();
 
 		result = gnome_vfs_get_file_info (file_name, info, GNOME_VFS_FILE_INFO_DEFAULT);
 
 		if (info != NULL
 		    && info->mime_type != NULL
+		    && strstr (info->mime_type, "html") == NULL
 		    && strstr (info->mime_type, "xml") == NULL
 		    && strstr (info->mime_type, "rss") == NULL) {
 			invalid_mime_type = TRUE;
@@ -455,7 +478,7 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 							 "It may be the wrong URL, or the feed may be broken. "
 							 "Would you like Rhythmbox to attempt to use it anyway?"),
 							 file_name);
-			
+
 			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
 				invalid_mime_type = FALSE;
 
@@ -468,12 +491,12 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 		if (invalid_mime_type)
 			return FALSE;
 	}
-    
+
 	/* first download file by gnome_vfs for use gnome network configuration */
+	rb_debug ("reading podcast feed %s", file_name);
 	result = gnome_vfs_read_entire_file (file_name, &file_size, &buffer);
 	if (result != GNOME_VFS_OK)
 		return TRUE;
-	    
 
 	/* initializing parse */
 	sax_handler = g_new0 (xmlSAXHandler, 1);
@@ -486,9 +509,9 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 	ctx->in_unknown_elt = 0;
 	ctx->channel_data = data;
 	ctx->prop_value = g_string_sized_new (512);
-                            
-	ctxt = xmlCreateMemoryParserCtxt (buffer, file_size);
-	if (ctx == NULL) {
+
+	parser = xmlCreateMemoryParserCtxt (buffer, file_size);
+	if (parser == NULL) {
 		g_free (sax_handler);
 		g_free (buffer);
 		g_string_free (ctx->prop_value, TRUE);
@@ -496,14 +519,14 @@ rb_podcast_parse_load_feed (RBPodcastChannel *data,
 		return FALSE;
 	}
 
-	ctx->xmlctx = ctxt;
-	ctxt->userData = ctx;
-	ctxt->sax = sax_handler;
-	xmlParseDocument (ctxt);
+	ctx->xmlctx = parser;
+	parser->userData = ctx;
+	parser->sax = sax_handler;
+	xmlParseDocument (parser);
 
 	g_free (sax_handler);
-	ctxt->sax = NULL;
-	xmlFreeParserCtxt (ctxt);
+	parser->sax = NULL;
+	xmlFreeParserCtxt (parser);
 
 	g_free (buffer);
 	g_string_free (ctx->prop_value, TRUE);
@@ -534,6 +557,12 @@ rb_podcast_parse_date (const char *date_str)
 		result = strptime (date_str, "%a, %d %b %Y 0%T", &tm);
 	}
 
+	/* close-to-RFC 2822, but with no seconds */
+	if (result == NULL) {
+		memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%a, %d %b %Y %R", &tm);
+	}
+
 	/* format without weekday */
 	if (result == NULL) {
 		memset (&tm, 0, sizeof (struct tm));
@@ -552,6 +581,12 @@ rb_podcast_parse_date (const char *date_str)
 		result = strptime (date_str, "%Y-%m-%d %T", &tm);
 	}
 
+	/* ISO date like without timezone */
+	if (result == NULL) {
+	memset (&tm, 0, sizeof (struct tm));
+		result = strptime (date_str, "%Y-%m-%d", &tm);
+	}
+
 	/* Broken weekday short names */
 	if (result == NULL) {
 		char *tmp;
@@ -576,7 +611,7 @@ rb_podcast_parse_date (const char *date_str)
 		char *tmp;
 
 		memset (&tm, 0, sizeof (struct tm));
-	
+
 		/* match first part of time string */
 		result = strptime (date_str, "%a %b %d %T ", &tm);
 
@@ -589,20 +624,19 @@ rb_podcast_parse_date (const char *date_str)
 			tmp = result+n;
 
 			/* make sure there was at least one character that matched */
-			if ((tmp != NULL) && n > 0) 
+			if ((tmp != NULL) && n > 0)
 				/* remaining part must be the year */
 				result = strptime (tmp, "%Y", &tm);
-			else 
+			else
 				result = NULL;
 		}
 	}
 
 	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));	
 		rb_debug ("unable to convert date string %s", date_str);
 	}
 
-	return (uintmax_t) mktime (&tm);
+	return (uintmax_t) ( (result==NULL) ? 0 : mktime (&tm) );
 }
 
 static gulong
@@ -618,10 +652,10 @@ rb_podcast_parse_time (const char *time_str)
 		result = strptime (time_str, "%M:%S", &tm);
 	}
 	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));	
+		memset (&tm, 0, sizeof (struct tm));
 		rb_debug ("unable to convert duration string %s", time_str);
 	}
-	
+
 	return ((tm.tm_hour * 60 + tm.tm_min) * 60 + tm.tm_sec);
 }
 
@@ -629,7 +663,7 @@ void
 rb_podcast_parse_channel_free (RBPodcastChannel *data)
 {
 	g_return_if_fail (data != NULL);
-	
+
 	g_list_foreach (data->posts, (GFunc) rb_podcast_parse_item_free, NULL);
 	g_list_free (data->posts);
 	data->posts = NULL;
@@ -644,16 +678,16 @@ rb_podcast_parse_channel_free (RBPodcastChannel *data)
 	g_free (data->contact);
 	g_free (data->img);
 	g_free (data->copyright);
-	
+
 	g_free (data);
 	data = NULL;
 }
-		
-void 
+
+void
 rb_podcast_parse_item_free (RBPodcastItem *item)
 {
 	g_return_if_fail (item != NULL);
-	
+
 	g_free (item->title);
 	g_free (item->url);
 	g_free (item->description);
