@@ -67,6 +67,13 @@ static void rb_library_browser_views_changed (GConfClient *client,
 G_DEFINE_TYPE (RBLibraryBrowser, rb_library_browser, GTK_TYPE_HBOX)
 #define RB_LIBRARY_BROWSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_BROWSER, RBLibraryBrowserPrivate))
 
+typedef struct
+{
+	RBLibraryBrowser *widget;
+	int rebuild_prop_index;
+	int rebuild_idle_id;
+} RBLibraryBrowserRebuildData;
+
 typedef struct
 {
 	RhythmDB *db;
@@ -79,6 +86,8 @@ typedef struct
 
 	GHashTable *property_views;
 	GHashTable *selections;
+
+	RBLibraryBrowserRebuildData *rebuild_data;
 } RBLibraryBrowserPrivate;
 
 enum
@@ -307,7 +316,7 @@ update_browser_property_visibilty (RhythmDBPropType prop,
 			gtk_widget_show (GTK_WIDGET (view));
 		} else {
 			gtk_widget_hide (GTK_WIDGET (view));
-			rb_property_view_reset (view);
+			rb_property_view_set_selection (view, NULL);
 		}
 	}
 }
@@ -329,6 +338,7 @@ update_browser_views_visibility (RBLibraryBrowser *widget)
 	}
 
 	g_hash_table_foreach (priv->property_views, (GHFunc)update_browser_property_visibilty, properties);
+	g_list_free (properties);
 }
 
 static void
@@ -596,6 +606,34 @@ rebuild_child_model (RBLibraryBrowser *widget,
 	g_object_unref (base_model);
 }
 
+static gboolean
+idle_rebuild_model (RBLibraryBrowserRebuildData *data)
+{
+	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (data->widget);
+
+	priv->rebuild_data = NULL;
+	rebuild_child_model (data->widget, data->rebuild_prop_index, FALSE);
+	return FALSE;
+}
+
+static void
+destroy_idle_rebuild_model (RBLibraryBrowserRebuildData *data)
+{
+	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (data->widget);
+	RBPropertyView *view;
+	RhythmDBPropType prop_type;
+	
+	prop_type = browser_properties[data->rebuild_prop_index].type;
+	view = g_hash_table_lookup (priv->property_views, (gpointer)prop_type);
+	if (view != NULL) {
+		ignore_selection_changes (data->widget, view, FALSE);
+	}
+
+	priv->rebuild_data = NULL;
+	g_object_unref (data->widget);
+	g_free (data);
+}
+
 void
 rb_library_browser_set_selection (RBLibraryBrowser *widget,
 				  RhythmDBPropType type,
@@ -604,6 +642,8 @@ rb_library_browser_set_selection (RBLibraryBrowser *widget,
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	GList *old_selection;
 	RBPropertyView *view;
+	int rebuild_index;
+	RBLibraryBrowserRebuildData *rebuild_data;
 
 	old_selection = g_hash_table_lookup (priv->selections, (gpointer)type);
 
@@ -615,13 +655,33 @@ rb_library_browser_set_selection (RBLibraryBrowser *widget,
 	else
 		g_hash_table_remove (priv->selections, (gpointer)type);
 
-	view = g_hash_table_lookup (priv->property_views, (gpointer)type);
-	if (view)
-		ignore_selection_changes (widget, view, TRUE);
+	rebuild_index = prop_to_index (type);
+	if (priv->rebuild_data != NULL) {
+		rebuild_data = priv->rebuild_data;
+		if (rebuild_data->rebuild_prop_index <= rebuild_index) {
+			/* already rebuilding a model further up the chain,
+			 * so we don't need to do anything for this one.
+			 */
+			return;
+		}
+		g_source_remove (rebuild_data->rebuild_idle_id);
+		rebuild_data = NULL;
+	}
 
-	rebuild_child_model (widget, prop_to_index (type), FALSE);
-	if (view)
-		ignore_selection_changes (widget, view, FALSE);
+	view = g_hash_table_lookup (priv->property_views, (gpointer)type);
+	if (view) {
+		ignore_selection_changes (widget, view, TRUE);
+	}
+
+	rebuild_data = g_new0 (RBLibraryBrowserRebuildData, 1);
+	rebuild_data->widget = g_object_ref (widget);
+	rebuild_data->rebuild_prop_index = rebuild_index;
+	rebuild_data->rebuild_idle_id =
+		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+				 (GSourceFunc) idle_rebuild_model,
+				 rebuild_data,
+				 (GDestroyNotify) destroy_idle_rebuild_model);
+	priv->rebuild_data = rebuild_data;
 }
 
 GList*
